# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Audio(Component):
    def __init__(
        self,
        *slots,
        autoplay=None,
        controls=None,
        crossorigin=None,
        anonymous=None,
        use_credentials=None,
        currentTime=None,
        loop=None,
        muted=None,
        preload=None,
        src=None
    ):
        self.attr(
            {
                "autoplay": autoplay,
                "controls": controls,
                "crossorigin": crossorigin,
                "anonymous": anonymous,
                "use-credentials": use_credentials,
                "currentTime": currentTime,
                "loop": loop,
                "muted": muted,
                "preload": preload,
                "src": src,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="audio").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Textarea(Component):
    def __init__(
        self,
        *slots,
        autocomplete=None,
        autofocus=None,
        cols=None,
        disabled=None,
        form=None,
        maxlength=None,
        minlength=None,
        name=None,
        placeholder=None,
        readonly=None,
        required=None,
        rows=None,
        spellcheck=None,
        wrap=None
    ):
        self.attr(
            {
                "autocomplete": autocomplete,
                "autofocus": autofocus,
                "cols": cols,
                "disabled": disabled,
                "form": form,
                "maxlength": maxlength,
                "minlength": minlength,
                "name": name,
                "placeholder": placeholder,
                "readonly": readonly,
                "required": required,
                "rows": rows,
                "spellcheck": spellcheck,
                "wrap": wrap,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="textarea").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class A(Component):
    def __init__(
        self,
        *slots,
        download=None,
        href=None,
        hreflang=None,
        ping=None,
        rel=None,
        target=None,
        type=None
    ):
        self.attr(
            {
                "download": download,
                "href": href,
                "hreflang": hreflang,
                "ping": ping,
                "rel": rel,
                "target": target,
                "type": type,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="a").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Embed(Component):
    def __init__(self, *slots, height=None, src=None, type=None, width=None):
        self.attr({"height": height, "src": src, "type": type, "width": width})

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="embed").proxy(self).render()



import bleach
from typing import Any, Dict
from .util import style_to_dict, dict_to_style


SELF_CLOSING = (
    "area",
    "base",
    "br",
    "col",
    "embed",
    "hr",
    "img",
    "input",
    "link",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
)


class Component:
    def __init__(
        self,
        *slots: "Component",
        tag_name: str = None,
        attrs: Dict[str, Any] = None,
        inner_html: str = "",
    ):
        self.styles = {}
        self.slots = slots

        self.tag_name = tag_name
        self.attrs = attrs or {}
        self.inner_html = inner_html

    def __repr__(self) -> str:
        return f"<Component '{self.__class__.__name__}'>"

    def proxy(self, other: "Component") -> "Component":
        self.style(**other.styles)
        self.attr(**other.attrs)
        self.inner_html += other.inner_html

        return self

    def style(self, **styles) -> "Component":
        self.styles.update({k: v for k, v in styles if v})

        return self

    def attr(self, **attrs) -> "Component":
        self.attrs.update({k: v for k, v in attrs if v})

        return self

    def safe_insert(self, insert: str) -> "Component":
        self.inner_html += bleach.clean(insert)

        return self

    def render(self) -> str:
        if not self.tag_name and self.inner_html:
            return self.inner_html

        if not self.tag_name:
            self.tag_name = "div"

        attrs = ""
        styles = {}
        style_attr = {}

        if self.styles or self.attrs.get("styles"):
            styles = {**self.styles, **style_to_dict(self.attrs.get("style"))}
            style_attr = {"style": dict_to_style(styles)}

        if self.attrs or self.styles:
            attrs = " " + " ".join(
                [
                    '{}="{}"'.format(k, v)
                    for k, v in {**self.attrs, **style_attr}.items()
                ]
            )

        if self.tag_name in SELF_CLOSING:
            return f"<{self.tag_name}{attrs}>"
        else:
            self.inner_html += "".join([c.render() for c in self.slots])

            return f"<{self.tag_name}{attrs}>{self.inner_html}</{self.tag_name}>"



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Body(Component):
    def __init__(
        self,
        *slots,
        onafterprint=None,
        onbeforeprint=None,
        onbeforeunload=None,
        onblur=None,
        onerror=None,
        onfocus=None,
        onhashchange=None,
        onload=None,
        onmessage=None,
        onoffline=None,
        ononline=None,
        onpopstate=None,
        onredo=None,
        onresize=None,
        onstorage=None,
        onundo=None,
        onunload=None
    ):
        self.attr(
            {
                "onafterprint": onafterprint,
                "onbeforeprint": onbeforeprint,
                "onbeforeunload": onbeforeunload,
                "onblur": onblur,
                "onerror": onerror,
                "onfocus": onfocus,
                "onhashchange": onhashchange,
                "onload": onload,
                "onmessage": onmessage,
                "onoffline": onoffline,
                "ononline": ononline,
                "onpopstate": onpopstate,
                "onredo": onredo,
                "onresize": onresize,
                "onstorage": onstorage,
                "onundo": onundo,
                "onunload": onunload,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="body").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Track(Component):
    def __init__(
        self, *slots, default=None, kind=None, label=None, src=None, srclang=None
    ):
        self.attr(
            {
                "default": default,
                "kind": kind,
                "label": label,
                "src": src,
                "srclang": srclang,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="track").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Link(Component):
    def __init__(
        self,
        *slots,
        as_=None,
        crossorigin=None,
        anonymous=None,
        use_credentials=None,
        disabled=None,
        href=None,
        hreflang=None,
        imagesizes=None,
        imagesrcset=None,
        media=None,
        rel=None,
        sizes=None,
        title=None,
        type=None
    ):
        self.attr(
            {
                "as": as_,
                "crossorigin": crossorigin,
                "anonymous": anonymous,
                "use-credentials": use_credentials,
                "disabled": disabled,
                "href": href,
                "hreflang": hreflang,
                "imagesizes": imagesizes,
                "imagesrcset": imagesrcset,
                "media": media,
                "rel": rel,
                "sizes": sizes,
                "title": title,
                "type": type,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="link").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Meta(Component):
    def __init__(self, *slots, charset=None, content=None, http_equiv=None, name=None):
        self.attr(
            {
                "charset": charset,
                "content": content,
                "http-equiv": http_equiv,
                "name": name,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="meta").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Col(Component):
    def __init__(self, *slots, span=None):
        self.attr({"span": span})

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="col").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Style(Component):
    def __init__(self, *slots, type=None, media=None, nonce=None, title=None):
        self.attr({"type": type, "media": media, "nonce": nonce, "title": title})

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="style").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Select(Component):
    def __init__(
        self,
        *slots,
        autocomplete=None,
        autofocus=None,
        disabled=None,
        form=None,
        multiple=None,
        name=None,
        required=None,
        size=None
    ):
        self.attr(
            {
                "autocomplete": autocomplete,
                "autofocus": autofocus,
                "disabled": disabled,
                "form": form,
                "multiple": multiple,
                "name": name,
                "required": required,
                "size": size,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="select").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Caption(Component):
    def __init__(self, *slots, left=None, top=None, right=None, bottom=None):
        self.attr({"left": left, "top": top, "right": right, "bottom": bottom})

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="caption").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Iframe(Component):
    def __init__(
        self,
        *slots,
        allow=None,
        allowfullscreen=None,
        allowpaymentrequest=None,
        height=None,
        name=None,
        referrerpolicy=None,
        sandbox=None,
        src=None,
        srcdoc=None,
        width=None
    ):
        self.attr(
            {
                "allow": allow,
                "allowfullscreen": allowfullscreen,
                "allowpaymentrequest": allowpaymentrequest,
                "height": height,
                "name": name,
                "referrerpolicy": referrerpolicy,
                "sandbox": sandbox,
                "src": src,
                "srcdoc": srcdoc,
                "width": width,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="iframe").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Source(Component):
    def __init__(
        self, *slots, media=None, sizes=None, src=None, srcset=None, type=None
    ):
        self.attr(
            {"media": media, "sizes": sizes, "src": src, "srcset": srcset, "type": type}
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="source").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Script(Component):
    def __init__(
        self,
        *slots,
        async_=None,
        crossorigin=None,
        defer=None,
        integrity=None,
        nomodule=None,
        nonce=None,
        referrerpolicy=None,
        src=None,
        type=None
    ):
        self.attr(
            {
                "async": async_,
                "crossorigin": crossorigin,
                "defer": defer,
                "integrity": integrity,
                "nomodule": nomodule,
                "nonce": nonce,
                "referrerpolicy": referrerpolicy,
                "src": src,
                "type": type,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="script").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Object(Component):
    def __init__(
        self,
        *slots,
        data=None,
        form=None,
        height=None,
        name=None,
        type=None,
        typemustmatch=None,
        usemap=None,
        width=None
    ):
        self.attr(
            {
                "data": data,
                "form": form,
                "height": height,
                "name": name,
                "type": type,
                "typemustmatch": typemustmatch,
                "usemap": usemap,
                "width": width,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="object").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Th(Component):
    def __init__(
        self, *slots, abbr=None, colspan=None, headers=None, rowspan=None, scope=None
    ):
        self.attr(
            {
                "abbr": abbr,
                "colspan": colspan,
                "headers": headers,
                "rowspan": rowspan,
                "scope": scope,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="th").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Td(Component):
    def __init__(self, *slots, colspan=None, headers=None, rowspan=None):
        self.attr({"colspan": colspan, "headers": headers, "rowspan": rowspan})

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="td").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Img(Component):
    def __init__(
        self,
        *slots,
        alt=None,
        crossorigin=None,
        anonymous=None,
        use_credentials=None,
        decoding=None,
        sync=None,
        async_=None,
        auto=None,
        height=None,
        ismap=None,
        loading=None,
        sizes=None,
        src=None,
        srcset=None,
        width=None,
        usemap=None
    ):
        self.attr(
            {
                "alt": alt,
                "crossorigin": crossorigin,
                "anonymous": anonymous,
                "use-credentials": use_credentials,
                "decoding": decoding,
                "sync": sync,
                "async": async_,
                "auto": auto,
                "height": height,
                "ismap": ismap,
                "loading": loading,
                "sizes": sizes,
                "src": src,
                "srcset": srcset,
                "width": width,
                "usemap": usemap,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="img").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Meter(Component):
    def __init__(
        self,
        *slots,
        value=None,
        min=None,
        max=None,
        low=None,
        high=None,
        optimum=None,
        form=None
    ):
        self.attr(
            {
                "value": value,
                "min": min,
                "max": max,
                "low": low,
                "high": high,
                "optimum": optimum,
                "form": form,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="meter").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Input(Component):
    def __init__(
        self,
        *slots,
        accept=None,
        alt=None,
        autocomplete=None,
        autofocus=None,
        capture=None,
        checked=None,
        dirname=None,
        disabled=None,
        form=None,
        formaction=None,
        formenctype=None,
        formmethod=None,
        formnovalidate=None,
        formtarget=None,
        height=None,
        id=None,
        inputmode=None,
        list=None,
        max=None,
        maxlength=None,
        min=None,
        minlength=None,
        multiple=None,
        name=None,
        pattern=None,
        placeholder=None,
        readonly=None,
        required=None,
        size=None,
        src=None,
        step=None,
        tabindex=None,
        title=None,
        type=None,
        value=None,
        width=None
    ):
        self.attr(
            {
                "accept": accept,
                "alt": alt,
                "autocomplete": autocomplete,
                "autofocus": autofocus,
                "capture": capture,
                "checked": checked,
                "dirname": dirname,
                "disabled": disabled,
                "form": form,
                "formaction": formaction,
                "formenctype": formenctype,
                "formmethod": formmethod,
                "formnovalidate": formnovalidate,
                "formtarget": formtarget,
                "height": height,
                "id": id,
                "inputmode": inputmode,
                "list": list,
                "max": max,
                "maxlength": maxlength,
                "min": min,
                "minlength": minlength,
                "multiple": multiple,
                "name": name,
                "pattern": pattern,
                "placeholder": placeholder,
                "readonly": readonly,
                "required": required,
                "size": size,
                "src": src,
                "step": step,
                "tabindex": tabindex,
                "title": title,
                "type": type,
                "value": value,
                "width": width,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="input").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Base(Component):
    def __init__(self, *slots, href=None, target=None):
        self.attr({"href": href, "target": target})

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="base").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Pre(Component):
    def __init__(
        self,
        *slots,
    ):

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="pre").proxy(self).render()



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Form(Component):
    def __init__(
        self, *slots, accept_charset=None, autocomplete=None, name=None, rel=None
    ):
        self.attr(
            {
                "accept-charset": accept_charset,
                "autocomplete": autocomplete,
                "name": name,
                "rel": rel,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="form").proxy(self).render()



from .template import Template  # noqa
from .slot import Slot  # noqa
from .summary import Summary  # noqa
from .menu import Menu  # noqa
from .dialog import Dialog  # noqa
from .details import Details  # noqa
from .textarea import Textarea  # noqa
from .select import Select  # noqa
from .progress import Progress  # noqa
from .output import Output  # noqa
from .option import Option  # noqa
from .optgroup import Optgroup  # noqa
from .meter import Meter  # noqa
from .legend import Legend  # noqa
from .label import Label  # noqa
from .input import Input  # noqa
from .form import Form  # noqa
from .fieldset import Fieldset  # noqa
from .datalist import Datalist  # noqa
from .button import Button  # noqa
from .tr import Tr  # noqa
from .thead import Thead  # noqa
from .th import Th  # noqa
from .tfoot import Tfoot  # noqa
from .td import Td  # noqa
from .tbody import Tbody  # noqa
from .table import Table  # noqa
from .colgroup import Colgroup  # noqa
from .col import Col  # noqa
from .caption import Caption  # noqa
from .ins import Ins  # noqa
from .del_ import Del  # noqa
from .script import Script  # noqa
from .noscript import Noscript  # noqa
from .canvas import Canvas  # noqa
from .source import Source  # noqa
from .picture import Picture  # noqa
from .param import Param  # noqa
from .object import Object  # noqa
from .iframe import Iframe  # noqa
from .embed import Embed  # noqa
from .video import Video  # noqa
from .track import Track  # noqa
from .map import Map  # noqa
from .img import Img  # noqa
from .audio import Audio  # noqa
from .area import Area  # noqa
from .wbr import Wbr  # noqa
from .var import Var  # noqa
from .u import U  # noqa
from .time import Time  # noqa
from .sup import Sup  # noqa
from .sub import Sub  # noqa
from .strong import Strong  # noqa
from .span import Span  # noqa
from .small import Small  # noqa
from .samp import Samp  # noqa
from .s import S  # noqa
from .ruby import Ruby  # noqa
from .rtc import Rtc  # noqa
from .rt import Rt  # noqa
from .rp import Rp  # noqa
from .rb import Rb  # noqa
from .q import Q  # noqa
from .mark import Mark  # noqa
from .kbd import Kbd  # noqa
from .i import I  # noqa
from .em import Em  # noqa
from .dfn import Dfn  # noqa
from .data import Data  # noqa
from .code import Code  # noqa
from .cite import Cite  # noqa
from .br import Br  # noqa
from .bdo import Bdo  # noqa
from .bdi import Bdi  # noqa
from .b import B  # noqa
from .abbr import Abbr  # noqa
from .a import A  # noqa
from .ul import Ul  # noqa
from .pre import Pre  # noqa
from .p import P  # noqa
from .ol import Ol  # noqa
from .li import Li  # noqa
from .hr import Hr  # noqa
from .figure import Figure  # noqa
from .figcaption import Figcaption  # noqa
from .dt import Dt  # noqa
from .dl import Dl  # noqa
from .div import Div  # noqa
from .dd import Dd  # noqa
from .blockquote import Blockquote  # noqa
from .section import Section  # noqa
from .nav import Nav  # noqa
from .main import Main  # noqa
from .hgroup import Hgroup  # noqa
from .header import Header  # noqa
from .footer import Footer  # noqa
from .aside import Aside  # noqa
from .article import Article  # noqa
from .address import Address  # noqa
from .body import Body  # noqa
from .title import Title  # noqa
from .style import Style  # noqa
from .meta import Meta  # noqa
from .link import Link  # noqa
from .head import Head  # noqa
from .base import Base  # noqa
from .html import Html  # noqa



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Tr(Component):
    def __init__(
        self, *slots, left=None, center=None, right=None, justify=None, char=None
    ):
        self.attr(
            {
                "left": left,
                "center": center,
                "right": right,
                "justify": justify,
                "char": char,
            }
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="tr").proxy(self).render()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['genjuice', 'genjuice.components', 'genjuice.components._gen']

package_data = \
{'': ['*']}

install_requires = \
['bleach>=3.2.1,<4.0.0']

setup_kwargs = {
    'name': 'genjuice',
    'version': '0.1.0',
    'description': 'A lightweight, component-based HTML builder',
    'long_description': '# GenJuice\n\nA lightweight, component-based HTML builder\n\n```py\nfrom genjuice import Component\nfrom genjuice.components import Button\n\n\nclass FancyButton(Component):\n    def render(self):\n        return Button(\n            "Hello, world!"\n        ).render()\n```\n',
    'author': 'docyx',
    'author_email': 'oliverxur@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/docyx/genjuice',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# NOTE: This is a generated file. Found a bug? Fix it in
# `scripts/generate_html_component`.
from ...component import Component


class Option(Component):
    def __init__(self, *slots, disabled=None, label=None, selected=None, value=None):
        self.attr(
            {"disabled": disabled, "label": label, "selected": selected, "value": value}
        )

        super().__init__(*slots)

    def render(self):
        return Component(tag_name="option").proxy(self).render()



# TODO: Make this all a class instead (`from_dict`, `from_str`, ...)?


def style_to_dict(style: str) -> dict:
    if not style:
        return {}

    dict_ = {}
    styles = [s for s in style.split(";") if s != ""]

    for style in styles:
        k, v = style.split(":")

        dict_[k.strip()] = v.strip()

    return dict_


def dict_to_style(dict_: dict) -> str:
    style = ""
    safe = lambda s: s.replace("_", "-")

    for k, v in dict_.items():
        style += f"{safe(k)}:{safe(v)};"

    return style
